import Blob from "cross-blob";
import { ChunkFile } from "../gamefile/chunk_file.js";
import { CollisionChunk } from "./collision.js";
export interface GridItem {
    load_id: number;
    collision_refs: GridCollisionRef[];
    fine_collision_refs: number[];
    breakable_refs: number[];
}
export interface GridCollisionRef {
    chunk_id: number;
    id: number;
    boundary_indices: number[];
}
export declare class GridChunk {
    grid: Array<GridItem | undefined>;
    width: number;
    height: number;
    x: number;
    z: number;
    scale: number;
    num_collision_chunks: number;
    constructor(grid?: Array<GridItem | undefined>, width?: number, height?: number, x?: number, z?: number, scale?: number, num_collision_chunks?: number);
    static from_blob(blob: Blob): Promise<GridChunk>;
    to_blob(): Blob;
    get_or_create_tile(int_x: number, int_z: number): GridItem;
    get_tile(int_x: number, int_z: number): GridItem | undefined;
    get_tiles_in_rect(minx: number, minz: number, maxx: number, maxz: number): [number, number][];
    expand_grid(expand_left: number, expand_up: number, expand_right: number, expand_down: number): void;
    /**
     * Rebuilds references to collision objects within this grid
     * @param file A chunk file containing the world with the relevant collision objects
     * @param do_trim Whether to trim the grid afterwards (See {@link trim})
     */
    rebuild(file: ChunkFile, do_trim?: boolean): Promise<void>;
    /**
     * Adds references to collision objects.
     * @param collision
     */
    add_collision(collision: CollisionChunk): void;
    /**
     * Removes references to collision objects with matching ID
     * @param collision Collision chunk or ID of collision chunk
     */
    remove_collision(collision: CollisionChunk | number): void;
    /**
     * Removes empty tiles from the grid, and shrinks the grid boundaries down
     * to fit the non-empty grid
     */
    trim(): void;
}
