import { VifCode } from "../ps2/vifcode.js";
import Blob from 'cross-blob';
export declare enum ModelNodeType {
    Empty = 0,
    Mesh = 1,
    ZoneGroup = 3,
    LodGroup = 4
}
export interface BufferList {
    vertices?: ArrayBuffer;
    normals?: ArrayBuffer;
    colors?: ArrayBuffer;
    kick_flags?: ArrayBuffer;
    uv?: ArrayBuffer;
    uv2?: ArrayBuffer;
    joints?: ArrayBuffer;
    weights?: ArrayBuffer;
    num_vertices: number;
    vertex_start: number;
}
interface ModelNodeBase {
    type: ModelNodeType;
    id: number;
    children: ModelNode[];
    bounds_origin: [number, number, number];
    bounds_size: [number, number, number];
    center: [number, number, number];
    addr?: number;
    radius: number;
    render_distance?: number;
}
export interface ModelNodeMesh extends ModelNodeBase {
    type: ModelNodeType.Mesh;
    material: number;
    initial_state: ArrayBuffer;
    vif_code: VifCode[];
}
export interface ModelNodeLodGroup extends ModelNodeBase {
    type: ModelNodeType.LodGroup;
    c1: ModelNode | null;
    c2: ModelNode | null;
    c3: ModelNode | null;
    fade_rate: number;
    render_distance: number;
    display_mask: number;
    sort_order: number;
}
export interface ModelNodeZoneGroup extends ModelNodeBase {
    type: ModelNodeType.ZoneGroup;
    zone_id: number;
}
export interface ModelNodeEmpty extends ModelNodeBase {
    type: ModelNodeType.Empty;
}
export declare type ModelNode = ModelNodeMesh | ModelNodeLodGroup | ModelNodeEmpty | ModelNodeZoneGroup;
export declare class ModelChunk {
    root: ModelNode;
    constructor(root?: ModelNode);
    to_blob(): Blob;
    static get_mesh_buffers(node: ModelNodeMesh): readonly [BufferList[], number];
    private allocate_node;
    private write_node;
    static from_blob(blob: Blob): Promise<ModelChunk>;
    private static read_node;
}
export {};
